@@ -8,14 +8,17 @@ import (
 	"net/url"
 	"os"
 	"strings"
+	"time"
 
 	"github.com/sagernet/sing-box/adapter"
 	"github.com/sagernet/sing-box/common/tls"
 	"github.com/sagernet/sing-box/option"
+	"github.com/sagernet/sing-box/transport/v2rayhttp"
 	"github.com/sagernet/sing/common"
 	E "github.com/sagernet/sing/common/exceptions"
 	M "github.com/sagernet/sing/common/metadata"
 	N "github.com/sagernet/sing/common/network"
+	aTLS "github.com/sagernet/sing/common/tls"
 	sHttp "github.com/sagernet/sing/protocol/http"
 
 	"golang.org/x/net/http2"
@@ -25,7 +28,8 @@ import (
 var _ adapter.V2RayServerTransport = (*Server)(nil)
 
 type Server struct {
-	handler      N.TCPConnectionHandler
+	tlsConfig    tls.ServerConfig
+	handler      adapter.V2RayServerTransportHandler
 	errorHandler E.Handler
 	httpServer   *http.Server
 	h2Server     *http2.Server
@@ -37,27 +41,19 @@ func (s *Server) Network() []string {
 	return []string{N.NetworkTCP}
 }
 
-func NewServer(ctx context.Context, options option.V2RayGRPCOptions, tlsConfig tls.ServerConfig, handler N.TCPConnectionHandler, errorHandler E.Handler) (*Server, error) {
+func NewServer(ctx context.Context, options option.V2RayGRPCOptions, tlsConfig tls.ServerConfig, handler adapter.V2RayServerTransportHandler) (*Server, error) {
 	server := &Server{
-		handler:      handler,
-		errorHandler: errorHandler,
-		path:         fmt.Sprintf("/%s/Tun", url.QueryEscape(options.ServiceName)),
-		h2Server:     new(http2.Server),
+		tlsConfig: tlsConfig,
+		handler:   handler,
+		path:      fmt.Sprintf("/%s/Tun", url.QueryEscape(options.ServiceName)),
+		h2Server: &http2.Server{
+			IdleTimeout: time.Duration(options.IdleTimeout),
+		},
 	}
 	server.httpServer = &http.Server{
 		Handler: server,
 	}
 	server.h2cHandler = h2c.NewHandler(server, server.h2Server)
-	if tlsConfig != nil {
-		stdConfig, err := tlsConfig.Config()
-		if err != nil {
-			return nil, err
-		}
-		if len(stdConfig.NextProtos) == 0 {
-			stdConfig.NextProtos = []string{http2.NextProtoTLS}
-		}
-		server.httpServer.TLSConfig = stdConfig
-	}
 	return server, nil
 }
 
@@ -67,19 +63,15 @@ func (s *Server) ServeHTTP(writer http.ResponseWriter, request *http.Request) {
 		return
 	}
 	if request.URL.Path != s.path {
-		request.Write(os.Stdout)
-		writer.WriteHeader(http.StatusNotFound)
-		s.badRequest(request, E.New("bad path: ", request.URL.Path))
+		s.fallbackRequest(request.Context(), writer, request, http.StatusNotFound, E.New("bad path: ", request.URL.Path))
 		return
 	}
 	if request.Method != http.MethodPost {
-		writer.WriteHeader(http.StatusNotFound)
-		s.badRequest(request, E.New("bad method: ", request.Method))
+		s.fallbackRequest(request.Context(), writer, request, http.StatusNotFound, E.New("bad method: ", request.Method))
 		return
 	}
 	if ct := request.Header.Get("Content-Type"); !strings.HasPrefix(ct, "application/grpc") {
-		writer.WriteHeader(http.StatusNotFound)
-		s.badRequest(request, E.New("bad content type: ", ct))
+		s.fallbackRequest(request.Context(), writer, request, http.StatusNotFound, E.New("bad content type: ", ct))
 		return
 	}
 	writer.Header().Set("Content-Type", "application/grpc")
@@ -87,28 +79,33 @@ func (s *Server) ServeHTTP(writer http.ResponseWriter, request *http.Request) {
 	writer.WriteHeader(http.StatusOK)
 	var metadata M.Metadata
 	metadata.Source = sHttp.SourceAddress(request)
-	conn := newGunConn(request.Body, writer, writer.(http.Flusher))
+	conn := v2rayhttp.NewHTTP2Wrapper(newGunConn(request.Body, writer, writer.(http.Flusher)))
 	s.handler.NewConnection(request.Context(), conn, metadata)
+	conn.CloseWrapper()
 }
 
-func (s *Server) badRequest(request *http.Request, err error) {
-	s.errorHandler.NewError(request.Context(), E.Cause(err, "process connection from ", request.RemoteAddr))
+func (s *Server) fallbackRequest(ctx context.Context, writer http.ResponseWriter, request *http.Request, statusCode int, err error) {
+	conn := v2rayhttp.NewHTTPConn(request.Body, writer)
+	fErr := s.handler.FallbackConnection(ctx, &conn, M.Metadata{})
+	if fErr == nil {
+		return
+	} else if fErr == os.ErrInvalid {
+		fErr = nil
+	}
+	if statusCode > 0 {
+		writer.WriteHeader(statusCode)
+	}
+	s.handler.NewError(request.Context(), E.Cause(E.Errors(err, E.Cause(fErr, "fallback connection")), "process connection from ", request.RemoteAddr))
 }
 
 func (s *Server) Serve(listener net.Listener) error {
-	fixTLSConfig := s.httpServer.TLSConfig == nil
-	err := http2.ConfigureServer(s.httpServer, s.h2Server)
-	if err != nil {
-		return err
-	}
-	if fixTLSConfig {
-		s.httpServer.TLSConfig = nil
-	}
-	if s.httpServer.TLSConfig == nil {
-		return s.httpServer.Serve(listener)
-	} else {
-		return s.httpServer.ServeTLS(listener, "", "")
+	if s.tlsConfig != nil {
+		if !common.Contains(s.tlsConfig.NextProtos(), http2.NextProtoTLS) {
+			s.tlsConfig.SetNextProtos(append([]string{"h2"}, s.tlsConfig.NextProtos()...))
+		}
+		listener = aTLS.NewListener(listener, s.tlsConfig)
 	}
+	return s.httpServer.Serve(listener)
 }
 
 func (s *Server) ServePacket(listener net.PacketConn) error {
