@@ -102,28 +102,40 @@ func (s *Server) ServeHTTP(writer http.ResponseWriter, request *http.Request) {
 		}
 	}
 
-	writer.WriteHeader(http.StatusOK)
-	writer.(http.Flusher).Flush()
-
 	var metadata M.Metadata
 	metadata.Source = sHttp.SourceAddress(request)
 	if h, ok := writer.(http.Hijacker); ok {
+		var requestBody *buf.Buffer
+		if contentLength := int(request.ContentLength); contentLength > 0 {
+			requestBody = buf.NewSize(contentLength)
+			_, err := requestBody.ReadFullFrom(request.Body, contentLength)
+			if err != nil {
+				s.fallbackRequest(request.Context(), writer, request, 0, E.Cause(err, "read request"))
+				return
+			}
+		}
+		writer.WriteHeader(http.StatusOK)
+		writer.(http.Flusher).Flush()
 		conn, reader, err := h.Hijack()
 		if err != nil {
-			s.fallbackRequest(request.Context(), writer, request, http.StatusInternalServerError, E.Cause(err, "hijack conn"))
+			s.fallbackRequest(request.Context(), writer, request, 0, E.Cause(err, "hijack conn"))
 			return
 		}
 		if cacheLen := reader.Reader.Buffered(); cacheLen > 0 {
 			cache := buf.NewSize(cacheLen)
 			_, err = cache.ReadFullFrom(reader.Reader, cacheLen)
 			if err != nil {
-				s.fallbackRequest(request.Context(), writer, request, http.StatusInternalServerError, E.Cause(err, "read cache"))
+				s.fallbackRequest(request.Context(), writer, request, 0, E.Cause(err, "read cache"))
 				return
 			}
 			conn = bufio.NewCachedConn(conn, cache)
 		}
+		if requestBody != nil {
+			conn = bufio.NewCachedConn(conn, requestBody)
+		}
 		s.handler.NewConnection(request.Context(), conn, metadata)
 	} else {
+		writer.WriteHeader(http.StatusOK)
 		conn := NewHTTP2Wrapper(&ServerHTTPConn{
 			NewHTTPConn(request.Body, writer),
 			writer.(http.Flusher),
@@ -141,12 +153,19 @@ func (s *Server) fallbackRequest(ctx context.Context, writer http.ResponseWriter
 	} else if fErr == os.ErrInvalid {
 		fErr = nil
 	}
-	writer.WriteHeader(statusCode)
+	if statusCode > 0 {
+		writer.WriteHeader(statusCode)
+	}
 	s.handler.NewError(request.Context(), E.Cause(E.Errors(err, E.Cause(fErr, "fallback connection")), "process connection from ", request.RemoteAddr))
 }
 
 func (s *Server) Serve(listener net.Listener) error {
 	if s.tlsConfig != nil {
+		if len(s.tlsConfig.NextProtos()) == 0 {
+			s.tlsConfig.SetNextProtos([]string{http2.NextProtoTLS, "http/1.1"})
+		} else if !common.Contains(s.tlsConfig.NextProtos(), http2.NextProtoTLS) {
+			s.tlsConfig.SetNextProtos(append([]string{"h2"}, s.tlsConfig.NextProtos()...))
+		}
 		listener = aTLS.NewListener(listener, s.tlsConfig)
 	}
 	return s.httpServer.Serve(listener)
